//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "fAddScaledFile.h"
#include "fMain.h"
#include <Graphics.hpp>

#include <sgConsts.hpp>
#include <DXF.hpp>
#include <DXFConv.hpp>

#include <CADImage.hpp>
#ifndef sgDXFONLY
#include <DWG.hpp>
#include <HPGL2.hpp>
#include <SVG.hpp>
#include <CGM.hpp>
#endif

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "DXF"
#ifndef sgDXFONLY
  #pragma link "DWG"
  #pragma link "HPGL2"
  #pragma link "SVG"
  #pragma link "CGM"
#endif
#pragma link "fValEdit"
#pragma resource "*.dfm"
TfmAddScaledFile *fmAddScaledFile;
//---------------------------------------------------------------------------
__fastcall TfmAddScaledFile::TfmAddScaledFile(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TfmAddScaledFile::SetImage(TsgCADImage * value)
{
  FCADFile = value;
}

TsgCADImage * __fastcall TfmAddScaledFile::GetImage()
{
  return FCADFile;
}

void __fastcall TfmAddScaledFile::SetFilesCount(int Value)
{
  FFilesCount = Value;
}

int __fastcall TfmAddScaledFile::GetFilesCount()
{
  return FFilesCount;
}

void __fastcall TfmAddScaledFile::ShowProp(TInsertProp * Prop)
{
  int I;
  FIsInternalChanging = true;
  AnsiString S;
  vePosX->Value = Ceil((double)(Prop->Position).X);
  vePosY->Value = Ceil((double)(Prop->Position).Y);
  vePosZ->Value = Ceil((double)(Prop->Position).Z);
  veScaleX->Value = Ceil((double)(Prop->Scale).X * 100.0);
  veScaleY->Value = Ceil((double)(Prop->Scale).Y * 100.0);
  veScaleZ->Value = Ceil((double)(Prop->Scale).Z * 100.0);
  veRotation->Value = Ceil(Prop->Rotation);
  if ((cbAddedFiles->Items->Count > 0)&&(cbAddedFiles->ItemIndex != -1))
  {
    I = cbAddedFiles->ItemIndex;
    S = IntToStr(cbAddedFiles->ItemIndex) + ". " + ExtractFileName(Prop->FileName);
    cbAddedFiles->Items->Strings[cbAddedFiles->ItemIndex] = S;
    cbAddedFiles->Text = S;
    cbAddedFiles->ItemIndex = I;
  }
  FIsInternalChanging = false;
}

void __fastcall TfmAddScaledFile::PropToStruct(TInsertProp* P)
{
  PInsertProp Prop;
  P->Position.X = vePosX->Value*1.0;
  P->Position.Y = vePosY->Value*1.0;
  P->Position.Z = vePosZ->Value*1.0;
  P->Scale.X = veScaleX->Value / 100.0;
  P->Scale.Y = veScaleY->Value / 100.0;
  P->Scale.Z = veScaleZ->Value / 100.0;
  P->Rotation = veRotation->Value*1.0;
  if ((cbAddedFiles->Items->Count > 0) && (cbAddedFiles->ItemIndex != -1))
  {
    Prop = (TInsertProp *)(cbAddedFiles->Items->Objects[cbAddedFiles->ItemIndex]);
    P->FileName = Prop->FileName;
  }
}

void __fastcall TfmAddScaledFile::btnApplyDefaultClick(TObject *Sender)
{
  vePosX->Value = 0;
  vePosY->Value = 0;
  vePosZ->Value = 0;
  veScaleX->Value = 100;
  veScaleY->Value = 100;
  veScaleZ->Value = 100;
  veRotation->Value = 0;
}

void __fastcall TfmAddScaledFile::btnOkClick(TObject *Sender)
{
  #ifndef sgDXFONLY
	AnsiString ExtsLine = "1.dxf2.dwg3.plt3.hg3.hgl3.rtl3.plo2.hp3.hp13.hp23.hpg3.hpgl3.hpgl23.gl23.prn3.spl4.svg4.svgz5.cgm";
  #endif

	int I, vPos;
	PInsertProp P;
	AnsiString S;
	TsgCADImage *vCADFile;

	try
	{
		FCADFile->ColorToLineWeight->Assign(fmMain->memColorToLineWeight->Lines);
		if (cbAddedFiles->Items->Count > 0)
		{
      FCADFile->IsWithoutBorder = cbWithoutBorder->Checked;
      for(I = 0; I <= cbAddedFiles->Items->Count - 1; I++)
      {
        P = (TInsertProp *)(cbAddedFiles->Items->Objects[I]);
        vCADFile = NULL;
        #ifndef sgDXFONLY
        vPos = AnsiPos(ExtractFileExt(P->FileName), ExtsLine);
        if (vPos > 1)
        {
          switch (ExtsLine[vPos-1]){
            case '1':
							vCADFile = new TsgCADdxfImage;
              break;
            case '2':
              vCADFile = new TsgDWGImage;
              break;
            case '3':
              vCADFile = new TsgHPGLImage;
              break;
            case '4':
              vCADFile = new TsgSVGImage;
              break;
            case '5':
              vCADFile = new TsgCGMImage;
              break;
          }
        }
        #else
				vCADFile = new TsgCADdxfImage;
        #endif
        if (vCADFile)
        {
          vCADFile->LoadFromFile(P->FileName);
          vCADFile->IsWithoutBorder = true;
          vCADFile->GetExtents();
          S = ChangeFileExt(ExtractFileName(P->FileName), "");
          FCADFile->AddScaledDXF(vCADFile, S, P->Position, P->Scale,
            P->Rotation);
        }
      }
		}
	}
  __finally
  {
    FCADFile->GetExtents();
    Close();
  }
}

void __fastcall TfmAddScaledFile::edtPropsChange(TObject *Sender)
{
  TInsertProp *P;
  TEdit *e;
  e = (TEdit *)Sender;
  if ((!FIsInternalChanging)&&(cbAddedFiles->ItemIndex >= 0)
	 &&(e->Text != "")&&(cbAddedFiles->Items->Count > 0))
  {
	P = (PInsertProp)(cbAddedFiles->Items->Objects[cbAddedFiles->ItemIndex]);
	PropToStruct(P);
  }
}

void __fastcall TfmAddScaledFile::btnOpenFileClick(TObject *Sender)
{
  AnsiString S;

  OpenDialog->FileName = "";
  if (!OpenDialog->Execute()) return;
  TInsertProp *P = new TInsertProp;
  PropToStruct(P);
  P->FileName = OpenDialog->FileName;
  S = IntToStr(FFilesCount)+". "+ExtractFileName(P->FileName);
  cbAddedFiles->Items->Add(S);
  cbAddedFiles->Items->Objects[cbAddedFiles->Items->Count - 1] = (TObject *)P;
  cbAddedFiles->ItemIndex = cbAddedFiles->Items->Count - 1;
  cbAddedFilesChange(Sender);
  FFilesCount++;
  lblCountWasAdded->Caption = IntToStr(FFilesCount);
  cbWithoutBorder->Enabled = false;
}

void __fastcall TfmAddScaledFile::cbAddedFilesChange(TObject *Sender)
{
  PInsertProp P;
  if ((cbAddedFiles->ItemIndex >= 0)&&(cbAddedFiles->Items->Count > 0))
  {
    P = (PInsertProp)(cbAddedFiles->Items->Objects[cbAddedFiles->ItemIndex]);
    ShowProp(P);
  }
}

void __fastcall TfmAddScaledFile::FormCreate(TObject *Sender)
{
	FCADFile = new TsgCADImage;
  OpenDialog->Filter = GraphicFilter(__classid(TGraphic));
  FFilesCount = 0;
  FIsInternalChanging = false;
}

void __fastcall TfmAddScaledFile::FormDestroy(TObject *Sender)
{
  int I;
  PInsertProp P;
  for(I=0;I<=cbAddedFiles->Items->Count - 1;I++)
  {
    P = (TInsertProp *)(cbAddedFiles->Items->Objects[I]);
    if (!(P)) delete P;
  }
  FCADFile->Free();  
}
//---------------------------------------------------------------------------
